# Client

[![CI](https://github.com/Relayz-io/client/actions/workflows/ci.yml/badge.svg?branch=develop)](https://github.com/Relayz-io/client/actions/workflows/ci.yml)

## Common-API 🦆

Commonly used **API** among different dependencies. For example **key-exchange-server**

## Web-Client 🦘

The web-client handles the exchange of keys between the meeting moderator and participants.
The client leverages blockchain's inherent security model by using it as a trust anchor (instead of any centralized authority or CA) for establishing secure channels between ```moderator``` and ```participant```. Each user in a blockchain has an ```ED25519``` PublicKey. In order to create a secure channel the **Diffie–Hellman** key exchange algorithm is used, provided by [Dalek cryptography](https://github.com/dalek-cryptography). Unfortunately, it's not possible to exchange keys with ```ED25519``` keys, because it uses an ```Edwards``` point, and the algorithm needs a ```Montgomery``` point. For this purpose, conversion happens [here](https://github.com/Relayz-io/near-client/blob/376708def420a158b3ca4e5af2aff2e380fd58af/src/crypto/dhx.rs#L168).

> NOTE! Our implementation uses the maximum key length available for each algorithm

Then we have proper keys for the **Diffie–Hellman** exchange. We derive the secret with [dhx](https://github.com/Relayz-io/key-exchange-client/blob/develop/web-client/src/crypto.rs#LL8C5-L8C5). Once that's been established we need to provide a uniformly distributed secret key. For this purpose we use the ***_KDF_*** algorithm [```blake3```](https://github.com/BLAKE3-team/BLAKE3). [Here](https://github.com/Relayz-io/key-exchange-client/blob/86450c015c2370742c765d94629ae97837224eee/web-client/src/crypto.rs#L18) is the source code pointer to our implementation.

The next stage is to distribute a randomly generated key securely to each participant. To accomplish this we encrypt the  _generated key_ with the ***_KDF_*** generated private key that is unique for each ```participant = moderator``` pair. We do this by encrypting the randomly generated key using [ChaCha20](https://rust-random.github.io/rand/rand_chacha/struct.ChaCha20Rng.html) with [AES-GCM](https://github.com/Relayz-io/key-exchange-client/blob/86450c015c2370742c765d94629ae97837224eee/web-client/src/crypto.rs#L30).

### Usage of web-client library:

Please follow the [official instructions](https://docs.github.com/en/enterprise-server@3.4/packages/working-with-a-github-packages-registry/working-with-the-npm-registry#installing-a-package) to use the latest package.

### Build

```sh
wasm-pack build web-client --target bundler --out-dir ../pkg
```

### Test

**Pre requirements:**
Clone a `key-exchange-server` and `near-smartcontracts`.
Deploy their docker instances:

⚠️ Please keep the order(`near-smartcontracts` depends on the `key-exchange-server` network)

1. `key-exchange-server`: ```bash docker-compose up -d```
2. `near-smartcontracts`: ```bash docker-compose -f docker-compose.yml -f docker-compose.tests.yml up -d```

Then:

`docker-compose up`
