use crate::error::ApiError;
use aes_gcm::{aead::Aead, Aes256Gcm, KeyInit, Nonce};
use near_client::{crypto::prelude::*, crypto::Error};
use rand::{RngCore, SeedableRng};
use rand_chacha::ChaChaRng;
use uuid::Uuid;

fn dhx(
    sk: &Ed25519SecretKey,
    other_pk: Ed25519PublicKey,
) -> Result<[u8; SECRET_KEY_LENGTH], Error> {
    let secret_key = SecretKey::try_from(Ed25519SecretKey::try_from_bytes(sk.as_bytes())?)?;
    let other_public_key = PublicKey::try_from(other_pk)?;

    Ok(secret_key.exchange(&other_public_key))
}

fn kdf(input: [u8; SECRET_KEY_LENGTH]) -> [u8; blake3::OUT_LEN] {
    // TODO: insert a commit hash to make it up to date
    blake3::derive_key("sample", &input)
}

pub fn secret() -> [u8; 32] {
    let mut chacha = ChaChaRng::from_entropy();
    let mut bytes = [0_u8; 32];
    chacha.fill_bytes(&mut bytes);
    bytes
}

pub fn encrypt(
    sk: &Ed25519SecretKey,
    other_pk: Ed25519PublicKey,
    _exchange_id: Uuid,
    secret: &[u8],
) -> Result<Vec<u8>, ApiError> {
    let bytes = dhx(sk, other_pk).map_err(ApiError::from)?;
    let secret_key = kdf(bytes);
    let aes = Aes256Gcm::new_from_slice(&secret_key)
        .map_err(|err| ApiError::AesKeyCreation(err.to_string()))?;
    let msg = aes
        .encrypt(Nonce::from_slice(b"unique nonce"), secret) // TODO: create unique nonce for each encrypt-decrypt pair
        .map_err(|err| ApiError::MessageEncryption(err.to_string()))?;

    Ok(msg)
}

pub fn decrypt(
    sk: &Ed25519SecretKey,
    other_pk: Ed25519PublicKey,
    _exchange_id: Uuid,
    msg: Vec<u8>,
) -> Result<Vec<u8>, ApiError> {
    let bytes = dhx(sk, other_pk).map_err(ApiError::from)?;
    let secret_key = kdf(bytes);
    let aes = Aes256Gcm::new_from_slice(&secret_key)
        .map_err(|err| ApiError::AesKeyCreation(err.to_string()))?;
    let secret = aes
        .decrypt(Nonce::from_slice(b"unique nonce"), msg.as_slice())
        .map_err(|err| ApiError::MessageEncryption(err.to_string()))?;

    Ok(secret)
}
